@@ -49,8 +49,7 @@ bool IsEGLConfigSupported(const PlatformParameters &param,
         angle::OpenSharedLibrary(ANGLE_EGL_LIBRARY_NAME, angle::SearchType::ModuleDir));
 #endif
 
-    EGLWindow *eglWindow =
-        EGLWindow::New(param.clientType, param.majorVersion, param.minorVersion, param.profileMask);
+    EGLWindow *eglWindow = EGLWindow::New(param.majorVersion, param.minorVersion);
     ConfigParameters configParams;
     bool result =
         eglWindow->initializeGL(osWindow, eglLibrary.get(), angle::GLESDriverType::AngleEGL,
@@ -76,8 +75,7 @@ bool IsSystemWGLConfigSupported(const PlatformParameters &param, OSWindow *osWin
     std::unique_ptr<angle::Library> openglLibrary(
         angle::OpenSharedLibrary("opengl32", angle::SearchType::SystemDir));
 
-    WGLWindow *wglWindow =
-        WGLWindow::New(param.clientType, param.majorVersion, param.minorVersion, param.profileMask);
+    WGLWindow *wglWindow = WGLWindow::New(param.majorVersion, param.minorVersion);
     ConfigParameters configParams;
     bool result =
         wglWindow->initializeGL(osWindow, openglLibrary.get(), angle::GLESDriverType::SystemWGL,
@@ -98,8 +96,7 @@ bool IsSystemEGLConfigSupported(const PlatformParameters &param, OSWindow *osWin
     eglLibrary.reset(OpenSharedLibraryWithExtension(GetNativeEGLLibraryNameWithExtension(),
                                                     SearchType::SystemDir));
 
-    EGLWindow *eglWindow =
-        EGLWindow::New(param.clientType, param.majorVersion, param.minorVersion, param.profileMask);
+    EGLWindow *eglWindow = EGLWindow::New(param.majorVersion, param.minorVersion);
     ConfigParameters configParams;
     bool result =
         eglWindow->initializeGL(osWindow, eglLibrary.get(), angle::GLESDriverType::SystemEGL,
@@ -131,20 +128,25 @@ bool IsAndroidDevice(const std::string &deviceName)
     return false;
 }
 
-bool IsAndroid9OrNewer()
+bool IsAndroidSdkLevelOrNewer(int level)
 {
     if (!IsAndroid())
     {
         return false;
     }
     SystemInfo *systemInfo = GetTestSystemInfo();
-    if (systemInfo->androidSdkLevel >= 28)
+    if (systemInfo->androidSdkLevel >= level)
     {
         return true;
     }
     return false;
 }
 
+bool IsAndroid9OrNewer()
+{
+    return IsAndroidSdkLevelOrNewer(28);
+}
+
 GPUDeviceInfo *GetActiveGPUDeviceInfo()
 {
     SystemInfo *systemInfo = GetTestSystemInfo();
@@ -196,11 +198,6 @@ bool IsMetalTextureSwizzleAvailable()
 {
     return false;
 }
-
-bool IsMetalCompressedTexture3DAvailable()
-{
-    return false;
-}
 #endif
 
 SystemInfo *GetTestSystemInfo()
@@ -309,11 +306,21 @@ bool IsPixel6()
     return IsAndroidDevice("Pixel 6");
 }
 
+bool IsGalaxyS22()
+{
+    return IsAndroidDevice("SM-S901B");
+}
+
 bool IsNVIDIAShield()
 {
     return IsAndroidDevice("SHIELD Android TV");
 }
 
+bool IsAndroid14OrNewer()
+{
+    return IsAndroidSdkLevelOrNewer(34);
+}
+
 bool IsIntel()
 {
     return HasSystemVendorID(kVendorID_Intel);
@@ -344,10 +351,19 @@ bool IsSwiftshaderDevice()
     return HasSystemDeviceID(kVendorID_GOOGLE, kDeviceID_Swiftshader);
 }
 
+bool IsSwiftShaderSupported()
+{
+#if defined(ANGLE_ENABLE_SWIFTSHADER)
+    return true;
+#else
+    return false;
+#endif
+}
+
 bool IsNVIDIA()
 {
 #if defined(ANGLE_PLATFORM_ANDROID)
-    // NVIDIA Shield cannot detect vendor ID (http://anglebug.com/3541)
+    // NVIDIA Shield cannot detect vendor ID (http://anglebug.com/42262205)
     if (IsNVIDIAShield())
     {
         return true;
@@ -358,19 +374,11 @@ bool IsNVIDIA()
 
 bool IsQualcomm()
 {
-    return IsNexus5X() || IsNexus9() || IsPixelXL() || IsPixel2() || IsPixel2XL() || IsPixel4() ||
+    return HasSystemVendorID(kVendorID_Qualcomm) || HasSystemVendorID(kVendorID_Qualcomm_DXGI) ||
+           IsNexus5X() || IsNexus9() || IsPixelXL() || IsPixel2() || IsPixel2XL() || IsPixel4() ||
            IsPixel4XL();
 }
 
-bool Is64Bit()
-{
-#if defined(ANGLE_IS_64_BIT_CPU)
-    return true;
-#else
-    return false;
-#endif  // defined(ANGLE_IS_64_BIT_CPU)
-}
-
 bool HasMesa()
 {
 #if defined(ANGLE_HAS_MESA)
@@ -394,26 +402,25 @@ bool IsConfigAllowlisted(const SystemInfo &systemInfo, const PlatformParameters
             return true;
     }
 
-    // TODO: http://crbug.com/swiftshader/145
-    // Swiftshader does not currently have all the robustness features
-    // we need for ANGLE. In particular, it is unable to detect and recover
-    // from infinitely looping shaders. That bug is the tracker for fixing
-    // that and when resolved we can remove the following code.
-    // This test will disable tests marked with the config WithRobustness
-    // when run with the swiftshader Vulkan driver and on Android.
-    if ((param.isSwiftshader() || IsSwiftshaderDevice()) &&
-        param.eglParameters.robustness == EGL_TRUE)
+    if (param.isSwiftshader() || IsSwiftshaderDevice())
     {
-        return false;
-    }
+        if (!IsSwiftShaderSupported())
+        {
+            return false;
+        }
 
-// Skip test configs that target the desktop OpenGL frontend when it's not enabled.
-#if !defined(ANGLE_ENABLE_GL_DESKTOP_FRONTEND)
-    if (param.isDesktopOpenGLFrontend())
-    {
-        return false;
+        // TODO: http://crbug.com/swiftshader/145
+        // Swiftshader does not currently have all the robustness features
+        // we need for ANGLE. In particular, it is unable to detect and recover
+        // from infinitely looping shaders. That bug is the tracker for fixing
+        // that and when resolved we can remove the following code.
+        // This test will disable tests marked with the config WithRobustness
+        // when run with the swiftshader Vulkan driver and on Android.
+        if (param.eglParameters.robustness == EGL_TRUE)
+        {
+            return false;
+        }
     }
-#endif
 
     if (param.driver == GLESDriverType::AngleVulkanSecondariesEGL)
     {
@@ -441,7 +448,7 @@ bool IsConfigAllowlisted(const SystemInfo &systemInfo, const PlatformParameters
                         return true;
                     case EGL_PLATFORM_ANGLE_TYPE_OPENGL_ANGLE:
                         // Note we disable AMD OpenGL testing on Windows due to using a very old and
-                        // outdated card with many driver bugs. See http://anglebug.com/5123
+                        // outdated card with many driver bugs. See http://anglebug.com/42263687
                         return !IsAMD();
                     case EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE:
                         if (IsARM64())
@@ -460,6 +467,8 @@ bool IsConfigAllowlisted(const SystemInfo &systemInfo, const PlatformParameters
 
                         // Win ES emulation is currently only supported on NVIDIA.
                         return IsNVIDIA(vendorID);
+                    case EGL_PLATFORM_ANGLE_TYPE_WEBGPU_ANGLE:
+                        return true;
                     default:
                         return false;
                 }
@@ -483,12 +492,19 @@ bool IsConfigAllowlisted(const SystemInfo &systemInfo, const PlatformParameters
         switch (param.getRenderer())
         {
             case EGL_PLATFORM_ANGLE_TYPE_OPENGL_ANGLE:
+                if (IsIOS())
+                {
+                    // OpenGL backend has been deprecated on iOS.
+                    return false;
+                }
                 // ES 3.1+ back-end is not supported properly.
                 if (param.majorVersion == 3 && param.minorVersion > 0)
                 {
                     return false;
                 }
                 return true;
+            case EGL_PLATFORM_ANGLE_TYPE_WEBGPU_ANGLE:
+                return true;
             case EGL_PLATFORM_ANGLE_TYPE_METAL_ANGLE:
                 if (!IsMetalRendererAvailable())
                 {
@@ -513,12 +529,12 @@ bool IsConfigAllowlisted(const SystemInfo &systemInfo, const PlatformParameters
         }
 
         // ES 3 configs do not work properly on Fuchsia ARM.
-        // TODO(anglebug.com/4352): Investigate missing features.
+        // TODO(anglebug.com/42262979): Investigate missing features.
         if (param.majorVersion > 2 && IsARM())
             return false;
 
         // Loading swiftshader is not brought up on Fuchsia.
-        // TODO(anglebug.com/4353): Support loading swiftshader vulkan ICD.
+        // TODO(anglebug.com/42262980): Support loading swiftshader vulkan ICD.
         if (param.getDeviceType() == EGL_PLATFORM_ANGLE_DEVICE_TYPE_SWIFTSHADER_ANGLE)
             return false;
 
@@ -560,15 +576,13 @@ bool IsConfigAllowlisted(const SystemInfo &systemInfo, const PlatformParameters
     {
         ASSERT(param.driver == GLESDriverType::AngleEGL);
 
-        // Currently we support the OpenGL and Vulkan back-ends on Linux.
+        // Currently we support the OpenGL, Vulkan and WebGPU back-ends on Linux.
         switch (param.getRenderer())
         {
             case EGL_PLATFORM_ANGLE_TYPE_OPENGL_ANGLE:
-                return true;
+            case EGL_PLATFORM_ANGLE_TYPE_WEBGPU_ANGLE:
             case EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE:
-                // http://issuetracker.google.com/173004081
-                return !IsIntel() || !param.isEnableRequested(Feature::AsyncCommandQueue) ||
-                       param.isDisableRequested(Feature::AsyncCommandQueue);
+                return true;
             default:
                 return false;
         }
@@ -602,10 +616,6 @@ bool IsConfigAllowlisted(const SystemInfo &systemInfo, const PlatformParameters
                 {
                     return false;
                 }
-                if (param.isDisableRequested(Feature::SupportsNegativeViewport))
-                {
-                    return false;
-                }
                 return true;
             default:
                 return false;
@@ -706,6 +716,12 @@ bool IsPlatformAvailable(const PlatformParameters &param)
 #else
             break;
 #endif
+        case EGL_PLATFORM_ANGLE_TYPE_WEBGPU_ANGLE:
+#if !defined(ANGLE_ENABLE_WGPU)
+            return false;
+#else
+            break;
+#endif
 
         default:
             std::cout << "Unknown test platform: " << param << std::endl;
