@@ -47,8 +47,10 @@
 #include "config.h"
 #include "RenderLayerScrollableArea.h"
 
+#include "AnchorPositionEvaluator.h"
 #include "Chrome.h"
 #include "DebugPageOverlays.h"
+#include "DocumentInlines.h"
 #include "Editor.h"
 #include "ElementRuleCollector.h"
 #include "EventHandler.h"
@@ -78,10 +80,13 @@
 #include "ScrollingCoordinator.h"
 #include "ShadowRoot.h"
 #include <wtf/SetForScope.h>
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/text/MakeString.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(RenderLayerScrollableArea);
+
 RenderLayerScrollableArea::RenderLayerScrollableArea(RenderLayer& layer)
     : m_layer(layer)
 {
@@ -98,23 +103,23 @@ void RenderLayerScrollableArea::clear()
 {
     auto& renderer = m_layer.renderer();
     if (m_registeredScrollableArea)
-        renderer.view().frameView().removeScrollableArea(this);
+        renderer.view().protectedFrameView()->removeScrollableArea(this);
     
     if (m_isRegisteredForAnimatedScroll) {
-        renderer.view().frameView().removeScrollableAreaForAnimatedScroll(this);
+        renderer.view().protectedFrameView()->removeScrollableAreaForAnimatedScroll(this);
         m_isRegisteredForAnimatedScroll = false;
     }
 
 #if ENABLE(IOS_TOUCH_EVENTS)
     unregisterAsTouchEventListenerForScrolling();
 #endif
-    if (auto* element = renderer.element())
+    if (RefPtr element = renderer.element())
         element->setSavedLayerScrollPosition(m_scrollPosition);
 
     destroyScrollbar(ScrollbarOrientation::Horizontal);
     destroyScrollbar(ScrollbarOrientation::Vertical);
 
-    if (auto* scrollingCoordinator = renderer.page().scrollingCoordinator())
+    if (RefPtr scrollingCoordinator = renderer.protectedPage()->scrollingCoordinator())
         scrollingCoordinator->willDestroyScrollableArea(*this);
 
     clearScrollCorner();
@@ -123,7 +128,7 @@ void RenderLayerScrollableArea::clear()
 
 void RenderLayerScrollableArea::restoreScrollPosition()
 {
-    auto* element = m_layer.renderer().element();
+    RefPtr element = m_layer.renderer().element();
     if (!element)
         return;
 
@@ -183,7 +188,7 @@ bool RenderLayerScrollableArea::isUserScrollInProgress() const
     if (!scrollsOverflow())
         return false;
 
-    if (auto scrollingCoordinator = m_layer.page().scrollingCoordinator()) {
+    if (RefPtr scrollingCoordinator = m_layer.protectedPage()->scrollingCoordinator()) {
         if (scrollingCoordinator->isUserScrollInProgress(scrollingNodeID()))
             return true;
     }
@@ -200,7 +205,7 @@ bool RenderLayerScrollableArea::isRubberBandInProgress() const
     if (!scrollsOverflow())
         return false;
 
-    if (auto scrollingCoordinator = m_layer.page().scrollingCoordinator()) {
+    if (RefPtr scrollingCoordinator = m_layer.protectedPage()->scrollingCoordinator()) {
         if (scrollingCoordinator->isRubberBandInProgress(scrollingNodeID()))
             return true;
     }
@@ -264,7 +269,7 @@ bool RenderLayerScrollableArea::requestScrollToPosition(const ScrollPosition& po
 #if ENABLE(ASYNC_SCROLLING)
     LOG_WITH_STREAM(Scrolling, stream << "RenderLayerScrollableArea::requestScrollToPosition " << position << " options  " << options);
 
-    if (auto* scrollingCoordinator = m_layer.page().scrollingCoordinator())
+    if (RefPtr scrollingCoordinator = m_layer.protectedPage()->scrollingCoordinator())
         return scrollingCoordinator->requestScrollToPosition(*this, position, options);
 #else
     UNUSED_PARAM(position);
@@ -275,7 +280,7 @@ bool RenderLayerScrollableArea::requestScrollToPosition(const ScrollPosition& po
 
 bool RenderLayerScrollableArea::requestStartKeyboardScrollAnimation(const KeyboardScroll& scrollData)
 {
-    if (auto* scrollingCoordinator = m_layer.page().scrollingCoordinator())
+    if (RefPtr scrollingCoordinator = m_layer.protectedPage()->scrollingCoordinator())
         return scrollingCoordinator->requestStartKeyboardScrollAnimation(*this, scrollData);
 
     return false;
@@ -283,7 +288,7 @@ bool RenderLayerScrollableArea::requestStartKeyboardScrollAnimation(const Keyboa
 
 bool RenderLayerScrollableArea::requestStopKeyboardScrollAnimation(bool immediate)
 {
-    if (auto* scrollingCoordinator = m_layer.page().scrollingCoordinator())
+    if (RefPtr scrollingCoordinator = m_layer.protectedPage()->scrollingCoordinator())
         return scrollingCoordinator->requestStopKeyboardScrollAnimation(*this, immediate);
 
     return false;
@@ -294,7 +299,7 @@ void RenderLayerScrollableArea::stopAsyncAnimatedScroll()
 #if ENABLE(ASYNC_SCROLLING)
     LOG_WITH_STREAM(Scrolling, stream << m_layer << " stopAsyncAnimatedScroll");
 
-    if (auto* scrollingCoordinator = m_layer.page().scrollingCoordinator())
+    if (RefPtr scrollingCoordinator = m_layer.protectedPage()->scrollingCoordinator())
         return scrollingCoordinator->stopAnimatedScroll(*this);
 #endif
 }
@@ -330,7 +335,7 @@ void RenderLayerScrollableArea::scrollTo(const ScrollPosition& position)
     if (!box)
         return;
 
-    LOG_WITH_STREAM(Scrolling, stream << "RenderLayerScrollableArea [" << scrollingNodeID() << "] scrollTo " << position << " from " << m_scrollPosition << " (is user scroll " << (currentScrollType() == ScrollType::User) << ")");
+    LOG_WITH_STREAM(Scrolling, stream << "RenderLayerScrollableArea [" << (scrollingNodeID() ? scrollingNodeID()->toString() : ""_str) << "] scrollTo " << position << " from " << m_scrollPosition << " (is user scroll " << (currentScrollType() == ScrollType::User) << ")");
 
     ScrollPosition newPosition = position;
     if (!box->isHTMLMarquee()) {
@@ -350,9 +355,10 @@ void RenderLayerScrollableArea::scrollTo(const ScrollPosition& position)
     }
 
     m_scrollPosition = newPosition;
+    m_layer.setSelfAndDescendantsNeedPositionUpdate();
 
     auto& renderer = m_layer.renderer();
-    if (auto* element = renderer.element())
+    if (RefPtr element = renderer.element())
         element->setSavedLayerScrollPosition(m_scrollPosition);
 
     RenderView& view = renderer.view();
@@ -360,10 +366,10 @@ void RenderLayerScrollableArea::scrollTo(const ScrollPosition& position)
     // Update the positions of our child layers (if needed as only fixed layers should be impacted by a scroll).
     // We don't update compositing layers, because we need to do a deep update from the compositing ancestor.
     if (!view.frameView().layoutContext().isInRenderTreeLayout()) {
-        // If we're in the middle of layout, we'll just update layers once layout has finished.
-        m_layer.updateLayerPositionsAfterOverflowScroll();
+        Style::AnchorPositionEvaluator::updateSnapshottedScrollOffsets(m_layer.renderer().protectedDocument());
 
-        view.frameView().scheduleUpdateWidgetPositions();
+        // If we're in the middle of layout, we'll just update layers once layout has finished.
+        view.protectedFrameView()->updateLayerPositionsAfterOverflowScroll(m_layer);
 
         if (!m_updatingMarqueePosition) {
             // Avoid updating compositing layers if, higher on the stack, we're already updating layer
@@ -383,21 +389,21 @@ void RenderLayerScrollableArea::scrollTo(const ScrollPosition& position)
         }
 
         // Update regions, scrolling may change the clip of a particular region.
-        renderer.document().invalidateRenderingDependentRegions();
-        DebugPageOverlays::didLayout(renderer.frame());
+        renderer.protectedDocument()->invalidateRenderingDependentRegions();
+        DebugPageOverlays::didLayout(renderer.protectedFrame());
     }
 
-    LocalFrame& frame = renderer.frame();
+    Ref frame = renderer.frame();
     CheckedPtr repaintContainer = renderer.containerForRepaint().renderer;
     // The caret rect needs to be invalidated after scrolling
-    frame.selection().setCaretRectNeedsUpdate();
+    frame->selection().setCaretRectNeedsUpdate();
 
     auto rectForRepaint = valueOrCompute(layer().cachedClippedOverflowRect(), [&] { return renderer.clippedOverflowRectForRepaint(repaintContainer.get()); });
 
     FloatQuad quadForFakeMouseMoveEvent = FloatQuad(rectForRepaint);
     if (repaintContainer)
         quadForFakeMouseMoveEvent = repaintContainer->localToAbsoluteQuad(quadForFakeMouseMoveEvent);
-    frame.eventHandler().dispatchFakeMouseMoveEventSoonInQuad(quadForFakeMouseMoveEvent);
+    frame->eventHandler().dispatchFakeMouseMoveEventSoonInQuad(quadForFakeMouseMoveEvent);
 
     bool requiresRepaint = true;
     if (usesCompositedScrolling()) {
@@ -425,14 +431,14 @@ void RenderLayerScrollableArea::scrollTo(const ScrollPosition& position)
     }
 
     // Schedule the scroll and scroll-related DOM events.
-    if (Element* element = renderer.element())
-        element->document().addPendingScrollEventTarget(*element);
+    if (RefPtr element = renderer.element())
+        element->protectedDocument()->addPendingScrollEventTarget(*element);
 
     if (scrollsOverflow())
-        view.frameView().didChangeScrollOffset();
+        view.protectedFrameView()->didChangeScrollOffset();
 
-    view.frameView().viewportContentsChanged();
-    frame.editor().renderLayerDidScroll(m_layer);
+    view.protectedFrameView()->viewportContentsChanged();
+    frame->protectedEditor()->renderLayerDidScroll(m_layer);
 }
 
 void RenderLayerScrollableArea::updateCompositingLayersAfterScroll()
@@ -518,10 +524,10 @@ void RenderLayerScrollableArea::setScrollOffset(const ScrollOffset& offset)
     scrollTo(scrollPositionFromOffset(offset));
 }
 
-ScrollingNodeID RenderLayerScrollableArea::scrollingNodeID() const
+std::optional<ScrollingNodeID> RenderLayerScrollableArea::scrollingNodeID() const
 {
     if (!m_layer.isComposited())
-        return { };
+        return std::nullopt;
 
     return m_layer.backing()->scrollingNodeIDForRole(ScrollCoordinationRole::Scrolling);
 }
@@ -533,8 +539,8 @@ bool RenderLayerScrollableArea::handleWheelEventForScrolling(const PlatformWheel
 
 #if ENABLE(ASYNC_SCROLLING)
     if (usesAsyncScrolling() && scrollingNodeID()) {
-        if (auto* scrollingCoordinator = m_layer.page().scrollingCoordinator()) {
-            auto result = scrollingCoordinator->handleWheelEventForScrolling(wheelEvent, scrollingNodeID(), gestureState);
+        if (RefPtr scrollingCoordinator = m_layer.protectedPage()->scrollingCoordinator()) {
+            auto result = scrollingCoordinator->handleWheelEventForScrolling(wheelEvent, *scrollingNodeID(), gestureState);
             if (!result.needsMainThreadProcessing())
                 return result.wasHandled;
         }
@@ -598,13 +604,13 @@ IntRect RenderLayerScrollableArea::convertFromScrollbarToContainingView(const Sc
     auto& renderer = m_layer.renderer();
     IntRect rect = scrollbarRect;
     rect.move(scrollbarOffset(scrollbar));
-    return renderer.view().frameView().convertFromRendererToContainingView(&renderer, rect);
+    return renderer.view().protectedFrameView()->convertFromRendererToContainingView(&renderer, rect);
 }
 
 IntRect RenderLayerScrollableArea::convertFromContainingViewToScrollbar(const Scrollbar& scrollbar, const IntRect& parentRect) const
 {
     auto& renderer = m_layer.renderer();
-    IntRect rect = renderer.view().frameView().convertFromContainingViewToRenderer(&renderer, parentRect);
+    IntRect rect = renderer.view().protectedFrameView()->convertFromContainingViewToRenderer(&renderer, parentRect);
     rect.move(-scrollbarOffset(scrollbar));
     return rect;
 }
@@ -614,13 +620,13 @@ IntPoint RenderLayerScrollableArea::convertFromScrollbarToContainingView(const S
     auto& renderer = m_layer.renderer();
     IntPoint point = scrollbarPoint;
     point.move(scrollbarOffset(scrollbar));
-    return renderer.view().frameView().convertFromRendererToContainingView(&renderer, point);
+    return renderer.view().protectedFrameView()->convertFromRendererToContainingView(&renderer, point);
 }
 
 IntPoint RenderLayerScrollableArea::convertFromContainingViewToScrollbar(const Scrollbar& scrollbar, const IntPoint& parentPoint) const
 {
     auto& renderer = m_layer.renderer();
-    IntPoint point = renderer.view().frameView().convertFromContainingViewToRenderer(&renderer, parentPoint);
+    IntPoint point = renderer.view().protectedFrameView()->convertFromContainingViewToRenderer(&renderer, parentPoint);
     point.move(-scrollbarOffset(scrollbar));
     return point;
 }
@@ -663,7 +669,7 @@ void RenderLayerScrollableArea::availableContentSizeChanged(AvailableSizeChangeR
 bool RenderLayerScrollableArea::shouldSuspendScrollAnimations() const
 {
     auto& renderer = m_layer.renderer();
-    return renderer.view().frameView().shouldSuspendScrollAnimations();
+    return renderer.view().protectedFrameView()->shouldSuspendScrollAnimations();
 }
 
 #if PLATFORM(IOS_FAMILY)
@@ -690,17 +696,19 @@ RenderLayer::OverflowControlRects RenderLayerScrollableArea::overflowControlsRec
     // Scrollbars sit inside the border box.
     auto overflowControlsPositioningRect = snappedIntRect(renderBox.paddingBoxRectIncludingScrollbar());
 
-    auto horizontalScrollbarHeight = m_hBar ? m_hBar->height() : 0;
-    auto verticalScrollbarWidth = m_vBar ? m_vBar->width() : 0;
+    RefPtr hBar = m_hBar;
+    RefPtr vBar = m_vBar;
+    auto horizontalScrollbarHeight = hBar ? hBar->height() : 0;
+    auto verticalScrollbarWidth = vBar ? vBar->width() : 0;
 
     auto isNonOverlayScrollbar = [](const Scrollbar* scrollbar) {
         return scrollbar && !scrollbar->isOverlayScrollbar();
     };
 
-    bool haveNonOverlayHorizontalScrollbar = isNonOverlayScrollbar(m_hBar.get());
-    bool haveNonOverlayVerticalScrollbar = isNonOverlayScrollbar(m_vBar.get());
+    bool haveNonOverlayHorizontalScrollbar = isNonOverlayScrollbar(hBar.get());
+    bool haveNonOverlayVerticalScrollbar = isNonOverlayScrollbar(vBar.get());
     bool placeVerticalScrollbarOnTheLeft = shouldPlaceVerticalScrollbarOnLeft();
-    bool haveResizer = renderBox.style().resize() != Resize::None;
+    bool haveResizer = renderBox.style().resize() != Resize::None && renderBox.style().pseudoElementType() == PseudoId::None;
     bool scrollbarsAvoidCorner = ((haveNonOverlayHorizontalScrollbar && haveNonOverlayVerticalScrollbar) || (haveResizer && (haveNonOverlayHorizontalScrollbar || haveNonOverlayVerticalScrollbar))) && renderBox.style().scrollbarWidth() != ScrollbarWidth::None;
 
     IntSize cornerSize;
@@ -714,7 +722,7 @@ RenderLayer::OverflowControlRects RenderLayerScrollableArea::overflowControlsRec
 
     RenderLayer::OverflowControlRects result;
 
-    if (m_hBar) {
+    if (hBar) {
         auto barRect = overflowControlsPositioningRect;
         barRect.shiftYEdgeTo(barRect.maxY() - horizontalScrollbarHeight);
         if (scrollbarsAvoidCorner) {
@@ -727,7 +735,7 @@ RenderLayer::OverflowControlRects RenderLayerScrollableArea::overflowControlsRec
         result.horizontalScrollbar = barRect;
     }
 
-    if (m_vBar) {
+    if (vBar) {
         auto barRect = overflowControlsPositioningRect;
         if (placeVerticalScrollbarOnTheLeft)
             barRect.setWidth(verticalScrollbarWidth);
@@ -783,12 +791,12 @@ void RenderLayerScrollableArea::invalidateScrollbarRect(Scrollbar& scrollbar, co
         return;
 
     if (&scrollbar == m_vBar.get()) {
-        if (GraphicsLayer* layer = layerForVerticalScrollbar()) {
+        if (RefPtr layer = layerForVerticalScrollbar()) {
             layer->setNeedsDisplayInRect(rect);
             return;
         }
     } else {
-        if (GraphicsLayer* layer = layerForHorizontalScrollbar()) {
+        if (RefPtr layer = layerForHorizontalScrollbar()) {
             layer->setNeedsDisplayInRect(rect);
             return;
         }
@@ -818,10 +826,8 @@ void RenderLayerScrollableArea::invalidateScrollCornerRect(const IntRect& rect)
     if (!showsOverflowControls())
         return;
 
-    if (GraphicsLayer* layer = layerForScrollCorner()) {
-        layer->setNeedsDisplayInRect(rect);
-        return;
-    }
+    if (layerForScrollCorner())
+        return ScrollableArea::invalidateScrollCorner(rect);
 
     if (m_scrollCorner)
         m_scrollCorner->repaintRectangle(rect);
@@ -831,14 +837,14 @@ void RenderLayerScrollableArea::invalidateScrollCornerRect(const IntRect& rect)
 
 NativeScrollbarVisibility RenderLayerScrollableArea::horizontalNativeScrollbarVisibility() const
 {
-    auto scrollbar = horizontalScrollbar();
-    return Scrollbar::nativeScrollbarVisibility(scrollbar);
+    RefPtr scrollbar = horizontalScrollbar();
+    return Scrollbar::nativeScrollbarVisibility(scrollbar.get());
 }
 
 NativeScrollbarVisibility RenderLayerScrollableArea::verticalNativeScrollbarVisibility() const
 {
-    auto scrollbar = verticalScrollbar();
-    return Scrollbar::nativeScrollbarVisibility(scrollbar);
+    RefPtr scrollbar = verticalScrollbar();
+    return Scrollbar::nativeScrollbarVisibility(scrollbar.get());
 }
 
 bool RenderLayerScrollableArea::canShowNonOverlayScrollbars() const
@@ -848,15 +854,15 @@ bool RenderLayerScrollableArea::canShowNonOverlayScrollbars() const
 
 void RenderLayerScrollableArea::createScrollbarsController()
 {
-    m_layer.page().chrome().client().ensureScrollbarsController(m_layer.page(), *this);
+    m_layer.page().chrome().client().ensureScrollbarsController(m_layer.protectedPage(), *this);
 }
 
 static inline RenderElement* rendererForScrollbar(RenderLayerModelObject& renderer)
 {
-    if (Element* element = renderer.element()) {
-        if (ShadowRoot* shadowRoot = element->containingShadowRoot()) {
+    if (RefPtr element = renderer.element()) {
+        if (RefPtr shadowRoot = element->containingShadowRoot()) {
             if (shadowRoot->mode() == ShadowRootMode::UserAgent)
-                return shadowRoot->host()->renderer();
+                return shadowRoot->protectedHost()->renderer();
         }
     }
 
@@ -878,10 +884,10 @@ Ref<Scrollbar> RenderLayerScrollableArea::createScrollbar(ScrollbarOrientation o
         widget = Scrollbar::createNativeScrollbar(*this, orientation, scrollbarWidthStyle());
         
         didAddScrollbar(widget.get(), orientation);
-        if (m_layer.page().isMonitoringWheelEvents())
-            scrollAnimator().setWheelEventTestMonitor(m_layer.page().wheelEventTestMonitor());
+        if (Ref page = m_layer.page(); page->isMonitoringWheelEvents())
+            scrollAnimator().setWheelEventTestMonitor(page->wheelEventTestMonitor());
     }
-    renderer.view().frameView().addChild(*widget);
+    renderer.view().protectedFrameView()->addChild(*widget);
     return widget.releaseNonNull();
 }
 
@@ -977,30 +983,32 @@ bool RenderLayerScrollableArea::hasScrollableOrRubberbandableAncestor()
 
 int RenderLayerScrollableArea::verticalScrollbarWidth(OverlayScrollbarSizeRelevancy relevancy, bool isHorizontalWritingMode) const
 {
-    if (m_vBar && m_vBar->isOverlayScrollbar() && (relevancy == IgnoreOverlayScrollbarSize || !m_vBar->shouldParticipateInHitTesting()))
+    RefPtr vBar = m_vBar;
+    if (vBar && vBar->isOverlayScrollbar() && (relevancy == OverlayScrollbarSizeRelevancy::IgnoreOverlayScrollbarSize || !vBar->shouldParticipateInHitTesting()))
         return 0;
 
-    if (!m_vBar && isHorizontalWritingMode && !(scrollbarGutterStyle().isAuto || ScrollbarTheme::theme().usesOverlayScrollbars()))
+    if (!vBar && isHorizontalWritingMode && !(scrollbarGutterStyle().isAuto || ScrollbarTheme::theme().usesOverlayScrollbars()))
         return ScrollbarTheme::theme().scrollbarThickness(scrollbarWidthStyle());
 
-    if (!m_vBar || !showsOverflowControls())
+    if (!vBar || !showsOverflowControls())
         return 0;
 
-    return m_vBar->width();
+    return vBar->width();
 }
 
 int RenderLayerScrollableArea::horizontalScrollbarHeight(OverlayScrollbarSizeRelevancy relevancy, bool isHorizontalWritingMode) const
 {
-    if (m_hBar && m_hBar->isOverlayScrollbar() && (relevancy == IgnoreOverlayScrollbarSize || !m_hBar->shouldParticipateInHitTesting()))
+    RefPtr hBar = m_hBar;
+    if (hBar && hBar->isOverlayScrollbar() && (relevancy == OverlayScrollbarSizeRelevancy::IgnoreOverlayScrollbarSize || !hBar->shouldParticipateInHitTesting()))
         return 0;
 
-    if (!m_hBar && !isHorizontalWritingMode && !(scrollbarGutterStyle().isAuto || ScrollbarTheme::theme().usesOverlayScrollbars()))
+    if (!hBar && !isHorizontalWritingMode && !(scrollbarGutterStyle().isAuto || ScrollbarTheme::theme().usesOverlayScrollbars()))
         return ScrollbarTheme::theme().scrollbarThickness(scrollbarWidthStyle());
 
-    if (!m_hBar || !showsOverflowControls())
+    if (!hBar || !showsOverflowControls())
         return 0;
 
-    return m_hBar->height();
+    return hBar->height();
 }
 
 OverscrollBehavior RenderLayerScrollableArea::horizontalOverscrollBehavior() const
@@ -1050,31 +1058,43 @@ bool RenderLayerScrollableArea::hasOverflowControls() const
     return m_hBar || m_vBar || m_scrollCorner || m_layer.renderer().style().resize() != Resize::None;
 }
 
-void RenderLayerScrollableArea::positionOverflowControls(const IntSize& offsetFromRoot)
+bool RenderLayerScrollableArea::positionOverflowControls(const IntSize& offsetFromRoot)
 {
     if (!m_hBar && !m_vBar && !m_layer.canResize())
-        return;
+        return false;
 
     if (!m_layer.renderBox())
-        return;
+        return false;
 
     auto rects = overflowControlsRects();
+    bool changed = false;
 
-    if (m_vBar) {
+    if (RefPtr vBar = m_vBar) {
         rects.verticalScrollbar.move(offsetFromRoot);
-        m_vBar->setFrameRect(rects.verticalScrollbar);
+        if (vBar->frameRect() != rects.verticalScrollbar) {
+            vBar->setFrameRect(rects.verticalScrollbar);
+            changed = true;
+        }
     }
 
-    if (m_hBar) {
+    if (RefPtr hBar = m_hBar) {
         rects.horizontalScrollbar.move(offsetFromRoot);
-        m_hBar->setFrameRect(rects.horizontalScrollbar);
+        if (hBar->frameRect() != rects.horizontalScrollbar) {
+            hBar->setFrameRect(rects.horizontalScrollbar);
+            changed = true;
+        }
     }
 
-    if (m_scrollCorner)
+    if (m_scrollCorner && m_scrollCorner->frameRect() != rects.scrollCorner) {
         m_scrollCorner->setFrameRect(rects.scrollCorner);
+        changed = true;
+    }
 
-    if (m_resizer)
+    if (m_resizer && m_resizer->frameRect() != rects.resizer) {
         m_resizer->setFrameRect(rects.resizer);
+        changed = true;
+    }
+    return changed;
 }
 
 LayoutUnit RenderLayerScrollableArea::overflowTop() const
@@ -1131,15 +1151,15 @@ void RenderLayerScrollableArea::computeScrollOrigin()
     ASSERT(box);
 
     int scrollableLeftOverflow = roundToInt(overflowLeft() - box->borderLeft());
-    if (shouldPlaceVerticalScrollbarOnLeft() /*|| box->style().blockFlowDirection() == BlockFlowDirection::RightToLeft*/)
-        scrollableLeftOverflow -= verticalScrollbarWidth(IgnoreOverlayScrollbarSize, box->style().isHorizontalWritingMode());
+    if (shouldPlaceVerticalScrollbarOnLeft())
+        scrollableLeftOverflow -= verticalScrollbarWidth(OverlayScrollbarSizeRelevancy::IgnoreOverlayScrollbarSize, box->writingMode().isHorizontal());
     int scrollableTopOverflow = roundToInt(overflowTop() - box->borderTop());
     setScrollOrigin(IntPoint(-scrollableLeftOverflow, -scrollableTopOverflow));
 
     // Horizontal scrollbar offsets depend on the scroll origin when vertical
     // scrollbars are on the left.
-    if (m_hBar)
-        m_hBar->offsetDidChange();
+    if (RefPtr hBar = m_hBar)
+        hBar->offsetDidChange();
 }
 
 void RenderLayerScrollableArea::computeHasCompositedScrollableOverflow(LayoutUpToDate layoutUpToDate)
@@ -1160,6 +1180,8 @@ void RenderLayerScrollableArea::computeHasCompositedScrollableOverflow(LayoutUpT
     if (hasCompositedScrollableOverflow == m_hasCompositedScrollableOverflow)
         return;
 
+    m_layer.setSelfAndDescendantsNeedPositionUpdate();
+
     // Whether this layer does composited scrolling affects the configuration of descendant sticky layers. We have to
     // dirty from the enclosing stacking context because overflow scroll doesn't create stacking context so those
     // containing block descendants may not be paint-order descendants, and the compositing dirty bits on RenderLayer act in paint order.
@@ -1232,12 +1254,12 @@ void RenderLayerScrollableArea::updateScrollbarPresenceAndState(std::optional<bo
     auto horizontalBarState = stateForScrollbarOnAxis(ScrollbarOrientation::Horizontal, hasHorizontalOverflow);
     setHasHorizontalScrollbar(horizontalBarState != ScrollbarState::NoScrollbar);
     if (horizontalBarState != ScrollbarState::NoScrollbar)
-        m_hBar->setEnabled(horizontalBarState == ScrollbarState::Enabled);
+        Ref { *m_hBar }->setEnabled(horizontalBarState == ScrollbarState::Enabled);
 
     auto verticalBarState = stateForScrollbarOnAxis(ScrollbarOrientation::Vertical, hasVerticalOverflow);
     setHasVerticalScrollbar(verticalBarState != ScrollbarState::NoScrollbar);
     if (verticalBarState != ScrollbarState::NoScrollbar)
-        m_vBar->setEnabled(verticalBarState == ScrollbarState::Enabled);
+        Ref { *m_vBar }->setEnabled(verticalBarState == ScrollbarState::Enabled);
 }
 
 void RenderLayerScrollableArea::updateScrollbarsAfterStyleChange(const RenderStyle* oldStyle)
@@ -1295,7 +1317,7 @@ void RenderLayerScrollableArea::updateScrollbarsAfterLayout()
                 renderer.setNeedsLayout(MarkOnlyThis);
                 if (CheckedPtr block = dynamicDowncast<RenderBlock>(renderer)) {
                     block->scrollbarsChanged(autoHorizontalScrollBarChanged, autoVerticalScrollBarChanged);
-                    block->layoutBlock(true);
+                    block->layoutBlock(RelayoutChildren::Yes);
                 } else
                     renderer.layout();
             }
@@ -1308,10 +1330,10 @@ void RenderLayerScrollableArea::updateScrollbarsAfterLayout()
     }
 
     // Set up the range.
-    if (m_hBar)
-        m_hBar->setProportion(roundToInt(box->clientWidth()), m_scrollWidth);
-    if (m_vBar)
-        m_vBar->setProportion(roundToInt(box->clientHeight()), m_scrollHeight);
+    if (RefPtr hBar = m_hBar)
+        hBar->setProportion(roundToInt(box->clientWidth()), m_scrollWidth);
+    if (RefPtr vBar = m_vBar)
+        vBar->setProportion(roundToInt(box->clientHeight()), m_scrollHeight);
 
     updateScrollbarSteps();
 
@@ -1327,13 +1349,13 @@ void RenderLayerScrollableArea::updateScrollbarSteps()
     paddedLayerBounds.contract(box->scrollPaddingForViewportRect(paddedLayerBounds));
 
     // Set up the  page step/line step.
-    if (m_hBar) {
+    if (RefPtr hBar = m_hBar) {
         int width = roundToInt(paddedLayerBounds.width());
-        m_hBar->setSteps(Scrollbar::pixelsPerLineStep(width), Scrollbar::pageStep(width));
+        hBar->setSteps(Scrollbar::pixelsPerLineStep(width), Scrollbar::pageStep(width));
     }
-    if (m_vBar) {
+    if (RefPtr vBar = m_vBar) {
         int height = roundToInt(paddedLayerBounds.height());
-        m_vBar->setSteps(Scrollbar::pixelsPerLineStep(height), Scrollbar::pageStep(height));
+        vBar->setSteps(Scrollbar::pixelsPerLineStep(height), Scrollbar::pageStep(height));
     }
 }
 
@@ -1443,7 +1465,7 @@ void RenderLayerScrollableArea::paintOverflowControls(GraphicsContext& context,
         if (!paintingRoot)
             paintingRoot = renderer.view().layer();
 
-        if (auto* scrollableArea = paintingRoot->scrollableArea())
+        if (CheckedPtr scrollableArea = paintingRoot->scrollableArea())
             scrollableArea->setContainsDirtyOverlayScrollbars(true);
         return;
     }
@@ -1462,10 +1484,10 @@ void RenderLayerScrollableArea::paintOverflowControls(GraphicsContext& context,
     positionOverflowControls(toIntSize(adjustedPaintOffset));
 
     // Now that we're sure the scrollbars are in the right place, paint them.
-    if (m_hBar && !layerForHorizontalScrollbar())
-        m_hBar->paint(context, damageRect);
-    if (m_vBar && !layerForVerticalScrollbar())
-        m_vBar->paint(context, damageRect);
+    if (RefPtr hBar = m_hBar; hBar && !layerForHorizontalScrollbar())
+        hBar->paint(context, damageRect);
+    if (RefPtr vBar = m_vBar; vBar && !layerForVerticalScrollbar())
+        vBar->paint(context, damageRect);
 
     if (layerForScrollCorner())
         return;
@@ -1506,7 +1528,8 @@ void RenderLayerScrollableArea::drawPlatformResizerImage(GraphicsContext& contex
     auto& renderer = m_layer.renderer();
     RefPtr<Image> resizeCornerImage;
     FloatSize cornerResizerSize;
-    if (renderer.document().deviceScaleFactor() >= 2) {
+    Ref document = renderer.document();
+    if (document->deviceScaleFactor() >= 2) {
         static NeverDestroyed<Image*> resizeCornerImageHiRes(&ImageAdapter::loadPlatformResource("textAreaResizeCorner@2x").leakRef());
         resizeCornerImage = resizeCornerImageHiRes;
         cornerResizerSize = resizeCornerImage->size();
@@ -1529,7 +1552,7 @@ void RenderLayerScrollableArea::drawPlatformResizerImage(GraphicsContext& contex
 
     if (!resizeCornerImage)
         return;
-    FloatRect imageRect = snapRectToDevicePixels(LayoutRect(resizerCornerRect.maxXMaxYCorner() - cornerResizerSize, cornerResizerSize), renderer.document().deviceScaleFactor());
+    FloatRect imageRect = snapRectToDevicePixels(LayoutRect(resizerCornerRect.maxXMaxYCorner() - cornerResizerSize, cornerResizerSize), document->deviceScaleFactor());
     context.drawImage(*resizeCornerImage, imageRect);
 }
 
@@ -1586,16 +1609,16 @@ bool RenderLayerScrollableArea::hitTestOverflowControls(HitTestResult& result, c
     }
 
     // FIXME: We should hit test the m_scrollCorner and pass it back through the result.
-    if (m_vBar && m_vBar->shouldParticipateInHitTesting()) {
+    if (RefPtr vBar = m_vBar; vBar && vBar->shouldParticipateInHitTesting()) {
         if (rects.verticalScrollbar.contains(localPoint)) {
-            result.setScrollbar(m_vBar.get());
+            result.setScrollbar(vBar.get());
             return true;
         }
     }
 
-    if (m_hBar && m_hBar->shouldParticipateInHitTesting()) {
+    if (RefPtr hBar = m_hBar; hBar && hBar->shouldParticipateInHitTesting()) {
         if (rects.horizontalScrollbar.contains(localPoint)) {
-            result.setScrollbar(m_hBar.get());
+            result.setScrollbar(hBar.get());
             return true;
         }
     }
@@ -1615,7 +1638,7 @@ bool RenderLayerScrollableArea::isActive() const
 
 IntPoint RenderLayerScrollableArea::lastKnownMousePositionInView() const
 {
-    return m_layer.renderer().view().frameView().lastKnownMousePositionInView();
+    return m_layer.renderer().view().protectedFrameView()->lastKnownMousePositionInView();
 }
 
 bool RenderLayerScrollableArea::isHandlingWheelEvent() const
@@ -1631,11 +1654,12 @@ bool RenderLayerScrollableArea::useDarkAppearance() const
 void RenderLayerScrollableArea::updateSnapOffsets()
 {
     // FIXME: Extend support beyond HTMLElements.
-    if (!is<HTMLElement>(m_layer.enclosingElement()) || !m_layer.enclosingElement()->renderBox())
+    RefPtr enclosingElement = m_layer.enclosingElement();
+    if (!is<HTMLElement>(enclosingElement) || !enclosingElement->renderBox())
         return;
 
-    RenderBox* box = m_layer.enclosingElement()->renderBox();
-    updateSnapOffsetsForScrollableArea(*this, *box, box->style(), box->paddingBoxRect(), box->style().writingMode(), box->style().direction(), m_layer.renderer().document().focusedElement());
+    RenderBox* box = enclosingElement->renderBox();
+    updateSnapOffsetsForScrollableArea(*this, *box, box->style(), box->paddingBoxRect(), box->style().writingMode(), m_layer.renderer().document().protectedFocusedElement().get());
 }
 
 bool RenderLayerScrollableArea::isScrollSnapInProgress() const
@@ -1643,7 +1667,7 @@ bool RenderLayerScrollableArea::isScrollSnapInProgress() const
     if (!scrollsOverflow())
         return false;
 
-    if (auto* scrollingCoordinator = m_layer.page().scrollingCoordinator()) {
+    if (RefPtr scrollingCoordinator = m_layer.protectedPage()->scrollingCoordinator()) {
         if (scrollingCoordinator->isScrollSnapInProgress(scrollingNodeID()))
             return true;
     }
@@ -1725,17 +1749,17 @@ bool RenderLayerScrollableArea::scrollingMayRevealBackground() const
 bool RenderLayerScrollableArea::isVisibleToHitTesting() const
 {
     auto& renderer = m_layer.renderer();
-    LocalFrameView& frameView = renderer.view().frameView();
-    return renderer.visibleToHitTesting() && frameView.isVisibleToHitTesting();
+    Ref frameView = renderer.view().frameView();
+    return renderer.visibleToHitTesting() && frameView->isVisibleToHitTesting();
 }
 
 void RenderLayerScrollableArea::updateScrollableAreaSet(bool hasOverflow)
 {
     auto& renderer = m_layer.renderer();
-    LocalFrameView& frameView = renderer.view().frameView();
+    Ref frameView = renderer.view().frameView();
 
     bool isVisibleToHitTest = renderer.visibleToHitTesting();
-    if (HTMLFrameOwnerElement* owner = frameView.frame().ownerElement())
+    if (RefPtr owner = frameView->frame().ownerElement())
         isVisibleToHitTest &= owner->renderer() && owner->renderer()->visibleToHitTesting();
 
     bool needsToBeRegistered = (hasOverflow && isVisibleToHitTest) || scrollAnimationStatus() == ScrollAnimationStatus::Animating;
@@ -1743,11 +1767,11 @@ void RenderLayerScrollableArea::updateScrollableAreaSet(bool hasOverflow)
 
     if (needsToBeRegistered) {
         if (!m_registeredScrollableArea) {
-            addedOrRemoved = frameView.addScrollableArea(this);
+            addedOrRemoved = frameView->addScrollableArea(this);
             m_registeredScrollableArea = true;
         }
     } else if (m_registeredScrollableArea) {
-        addedOrRemoved = frameView.removeScrollableArea(this);
+        addedOrRemoved = frameView->removeScrollableArea(this);
         m_registeredScrollableArea = false;
     }
 
@@ -1769,9 +1793,9 @@ void RenderLayerScrollableArea::updateScrollableAreaSet(bool hasOverflow)
 void RenderLayerScrollableArea::registerScrollableAreaForAnimatedScroll()
 {
     auto& renderer = m_layer.renderer();
-    LocalFrameView& frameView = renderer.view().frameView();
+    Ref frameView = renderer.view().frameView();
     if (!m_registeredScrollableArea) {
-        frameView.addScrollableAreaForAnimatedScroll(this);
+        frameView->addScrollableAreaForAnimatedScroll(this);
         m_isRegisteredForAnimatedScroll = true;
     }
 }
@@ -1788,7 +1812,7 @@ void RenderLayerScrollableArea::updateScrollCornerStyle()
     }
 
     if (!m_scrollCorner) {
-        m_scrollCorner = createRenderer<RenderScrollbarPart>(renderer.document(), WTFMove(*corner));
+        m_scrollCorner = createRenderer<RenderScrollbarPart>(renderer.protectedDocument(), WTFMove(*corner));
         // FIXME: A renderer should be a child of its parent!
         m_scrollCorner->setParent(&renderer);
         m_scrollCorner->initializeStyle();
@@ -1819,7 +1843,7 @@ void RenderLayerScrollableArea::updateResizerStyle()
     }
 
     if (!m_resizer) {
-        m_resizer = createRenderer<RenderScrollbarPart>(renderer.document(), WTFMove(*resizer));
+        m_resizer = createRenderer<RenderScrollbarPart>(renderer.protectedDocument(), WTFMove(*resizer));
         // FIXME: A renderer should be a child of its parent!
         m_resizer->setParent(&renderer);
         m_resizer->initializeStyle();
@@ -1928,7 +1952,7 @@ LayoutRect RenderLayerScrollableArea::scrollRectToVisible(const LayoutRect& abso
 
     auto revealRect = getRectToExposeForScrollIntoView(layerBounds, localExposeRect, options.alignX, options.alignY, localVisiblityRect);
     auto scrollPositionOptions = ScrollPositionChangeOptions::createProgrammatic();
-    if (!box->frame().eventHandler().autoscrollInProgress() && box->element() && useSmoothScrolling(options.behavior, box->element()))
+    if (!box->frame().eventHandler().autoscrollInProgress() && box->element() && useSmoothScrolling(options.behavior, box->protectedElement().get()))
         scrollPositionOptions.animated = ScrollIsAnimated::Yes;
     if (auto result = updateScrollPositionForScrollIntoView(scrollPositionOptions, revealRect, localExposeRect))
         return result.value();
@@ -1973,7 +1997,7 @@ void RenderLayerScrollableArea::scrollByRecursively(const IntSize& delta, Scroll
         if (!remainingScrollOffset.isZero() && renderer.parent()) {
             // FIXME: This skips scrollable frames.
             if (auto* enclosingScrollableLayer = m_layer.enclosingScrollableLayer(IncludeSelfOrNot::ExcludeSelf, CrossFrameBoundaries::Yes)) {
-                if (auto* scrollableArea = enclosingScrollableLayer->scrollableArea())
+                if (CheckedPtr scrollableArea = enclosingScrollableLayer->scrollableArea())
                     scrollableArea->scrollByRecursively(remainingScrollOffset, scrolledArea);
             }
 
@@ -1982,7 +2006,7 @@ void RenderLayerScrollableArea::scrollByRecursively(const IntSize& delta, Scroll
     } else {
         // If we are here, we were called on a renderer that can be programmatically scrolled, but doesn't
         // have an overflow clip. Which means that it is a document node that can be scrolled.
-        renderer.view().frameView().scrollBy(delta);
+        renderer.view().protectedFrameView()->scrollBy(delta);
         if (scrolledArea)
             *scrolledArea = &renderer.view().frameView();
 
@@ -1998,7 +2022,7 @@ bool RenderLayerScrollableArea::mockScrollbarsControllerEnabled() const
 
 void RenderLayerScrollableArea::logMockScrollbarsControllerMessage(const String& message) const
 {
-    m_layer.renderer().document().addConsoleMessage(MessageSource::Other, MessageLevel::Debug, makeString("RenderLayer: "_s, message));
+    m_layer.renderer().protectedDocument()->addConsoleMessage(MessageSource::Other, MessageLevel::Debug, makeString("RenderLayer: "_s, message));
 }
 
 String RenderLayerScrollableArea::debugDescription() const
@@ -2008,22 +2032,22 @@ String RenderLayerScrollableArea::debugDescription() const
 
 void RenderLayerScrollableArea::didStartScrollAnimation()
 {
-    m_layer.page().scheduleRenderingUpdate({ RenderingUpdateStep::Scroll });
+    m_layer.protectedPage()->scheduleRenderingUpdate({ RenderingUpdateStep::Scroll });
 }
 
 void RenderLayerScrollableArea::animatedScrollDidEnd()
 {
     if (m_isRegisteredForAnimatedScroll) {
         auto& renderer = m_layer.renderer();
-        LocalFrameView& frameView = renderer.view().frameView();
+        Ref frameView = renderer.view().frameView();
         m_isRegisteredForAnimatedScroll = false;
-        frameView.removeScrollableAreaForAnimatedScroll(this);
+        frameView->removeScrollableAreaForAnimatedScroll(this);
     }
 }
 
 float RenderLayerScrollableArea::deviceScaleFactor() const
 {
-    return m_layer.renderer().document().deviceScaleFactor();
+    return m_layer.renderer().protectedDocument()->deviceScaleFactor();
 }
 
 void RenderLayerScrollableArea::updateScrollAnchoringElement()
@@ -2044,7 +2068,7 @@ void RenderLayerScrollableArea::invalidateScrollAnchoringElement()
         m_scrollAnchoringController->invalidateAnchorElement();
 }
 
-FrameIdentifier RenderLayerScrollableArea::rootFrameID() const
+std::optional<FrameIdentifier> RenderLayerScrollableArea::rootFrameID() const
 {
     return m_layer.renderer().frame().rootFrame().frameID();
 }
